"""Contains dictionaries representing all the settings

The dictionaries are populated during initialization
"""

config = {}
gatherers = {}
inspectors = {}
loggers = {}
formatters = {}
environment = {}
